TrotterOrder

Enum TrotterOrder 

pub enum TrotterOrder {
    First,
    Second,
}
Trotter decomposition orders.

Variants



First

First-order Trotter decomposition.

For H = Σⱼ Hⱼ, approximates exp(-iHt) ≈ Πⱼ exp(-iHⱼt). This is the simplest and fastest method but has O(t²) error per step.



Second

Second-order Trotter decomposition.

Uses the symmetric Suzuki formula for improved accuracy with O(t³) error per step. Requires twice as many exponential operations as first-order.