pub enum Pauli {
X,
Y,
Z,
}Expand description
Defines the Pauli operators: X, Y, Z.
Variants
Implementations
Sourceimpl Pauli
impl Pauli
Sourcepub fn to_pauli_string(&self, target_qubit: usize) -> PauliString
pub fn to_pauli_string(&self, target_qubit: usize) -> PauliString
Returns the Pauli string representation of the operator.
Returns
- A Pauli string with the specified Pauli operator on the target qubit and a coefficient of 1.0.