pub struct RotateY { /* private fields */ }Expand description
Defines the rotate-Y operator
A single-qubit operator that applies a rotation around the Y axis of the Bloch sphere by the given angle. Also known as the RY gate.
Implementations