Unitary2

Struct Unitary2 

pub struct Unitary2 { /* private fields */ }
An arbitrary 2×2 unitary operator.

This operator can be applied to a single qubit in a quantum state. It is represented by a 2×2 unitary matrix. The matrix must be unitary. The operator can be constructed fallibly from a 2 x 2 unitary matrix, or infallibly from a rotation angle and phase shift angle.

Implementations

Source

impl Unitary2

pub fn new(matrix: [[Complex<f64>; 2]; 2]) -> Result<Self, Error>

Creates a new Unitary2 operator with the given 2×2 unitary matrix.

Arguments:
  • matrix - A 2×2 unitary matrix represented as a 2D array of complex numbers.
Returns:
  • Result<Self, Error> - A result containing the new Unitary2 operator or an error if the matrix is not unitary.
Errors:
  • Error::NonUnitaryMatrix - If the provided matrix is not unitary.
pub fn from_ry_phase(theta: f64, phi: f64) -> Self

Creates a new Unitary2 operator from a rotation angle theta and phase shift angle phi. This operator can be decomposed into a rotation around the Y axis followed by a phase shift. The enclosed unitary matrix is guaranteed to be unitary.

Special cases include:

  • U(theta, 0) = RY(theta)
  • U(0, phi) = PhaseShift(phi)
  • U(Pi/2, Pi) = Hadamard
  • U(Pi, Pi) = Pauli-X
Arguments:
  • theta - The rotation angle in radians.

  • phi - The phase shift angle in radians

Returns:
  • Self - A new Unitary2 operator representing the rotation and phase shift.
pub fn from_ry_phase_dagger(theta: f64, phi: f64) -> Self

Creates a new Unitary2 operator from a rotation angle theta and phase shift angle phi. This operator can be decomposed into a phase shift followed by a rotation around the Y axis. The enclosed unitary matrix is guaranteed to be unitary.

This is the adjoint of the ry_phase operator.

Arguments:
  • theta - The rotation angle in radians.

  • phi - The phase shift angle in radians.

Returns:
  • Self - A new Unitary2 operator representing the phase shift and rotation.