Module parameter

Module parameter 

Structs

Parameter
A struct representing a set of mutable parameters for a parametrised gate. Cloning this struct is cheap and allows multiple gates to share the same underlying parameters.