Parameter

Struct Parameter 

pub struct Parameter<const N: usize> { /* private fields */ }
A struct representing a set of mutable parameters for a parametrised gate. Cloning this struct is cheap and allows multiple gates to share the same underlying parameters.

Fields

  • values: An Arc<Mutex<[f64; N]>> that holds the parameter values.

Generic Parameters

  • N: The number of values in the parameter array.

Implementations

Source

impl<const N: usize> Parameter<N>

pub fn new(initial_values: [f64; N]) -> Self

Creates a new Parameter instance with the given initial values.

Arguments
  • initial_values - An array of initial parameter values.
Returns

A new Parameter instance.

pub fn deep_clone(&self) -> Self

Creates a new Parameter instance as an independent copy of the given parameter.

Returns

A new Parameter instance.

pub fn get(&self) -> [f64; N]

Gets the current values of the parameters. This will lock the underlying mutex.

Returns

The current parameter values.

If the mutex is poisoned, the function will return the current values.

pub fn set(&self, new_values: [f64; N])

Sets the values of the parameters. This will lock the underlying mutex.