pub struct ParametricRyPhase {
pub parameter: Parameter<2>,
}Expand description
A parametrised RY phase gate
This operator can be decomposed into a rotation around the Y axis followed by a phase shift. The enclosed unitary matrix is guaranteed to be unitary.
Fields
parameter: AParameter<2>instance that holds the rotation angle (theta) and phase shift (phi).
Fields
parameter: Parameter<2>Trait Implementations
Sourceimpl Clone for ParametricRyPhase
impl Clone for ParametricRyPhase
Sourcefn clone(&self) -> ParametricRyPhase
fn clone(&self) -> ParametricRyPhase
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for ParametricRyPhase
impl Debug for ParametricRyPhase
Sourceimpl ParametricGate for ParametricRyPhase
impl ParametricGate for ParametricRyPhase