Trait ParametricGate

pub trait ParametricGate:
    Debug
    + Send
    + Sync {
    // Required methods
    fn to_concrete_gates(
        &self,
        target_indices: &[usize],
        control_indices: &[usize],
    ) -> Vec<Gate>;
    fn box_clone(&self) -> Box<dyn ParametricGate>;
}
A trait for gates that can be parametrised.

The trait defines the to_concrete_gate method for creating a concrete gate from the parametrised gate. This is required for applying the parametrised gate in a circuit or compiling it into OpenQASM.

Required Methods

fn to_concrete_gates( &self, target_indices: &[usize], control_indices: &[usize], ) -> Vec<Gate>

Creates a concrete Gate using the current parameter values.

Arguments
  • target_indices - The indices of the target qubits.
  • control_indices - The indices of the control qubits.
Returns

Concrete Gate::Operator variants of the parameterised gate instance.

fn box_clone(&self) -> Box<dyn ParametricGate>

Clones the trait object. This is required by the Gate trait.