pub struct SumOp {
pub terms: Vec<PauliString>,
}Expand description
A vector of Pauli strings that are summed together. Useful for representing Hamiltonians or observables.
Fields
terms- A vector ofPauliStringinstances that are summed together.
Fields
terms: Vec<PauliString>A vector of Pauli strings that are summed together.
Implementations
Sourceimpl SumOp
impl SumOp
Sourcepub fn new(terms: Vec<PauliString>) -> Self
pub fn new(terms: Vec<PauliString>) -> Self
Sourcepub fn add_term(&mut self, term: PauliString)
pub fn add_term(&mut self, term: PauliString)
Sourcepub fn with_term(self, term: PauliString) -> Self
pub fn with_term(self, term: PauliString) -> Self
Sourcepub fn apply(&self, state: &State) -> Result<State, Error>
pub fn apply(&self, state: &State) -> Result<State, Error>
Applies the sum of Pauli strings to a given state.
Arguments
state- The state to which the sum of Pauli strings is applied.
Returns
Result<State, Error>- The resulting state after applying the sum of Pauli strings, or an error if the operation fails.
Errors
- Returns an error if the operations in the Pauli strings refer to qubits outside the range of the state.
Sourcepub fn expectation_value(&self, state: &State) -> Result<Complex<f64>, Error>
pub fn expectation_value(&self, state: &State) -> Result<Complex<f64>, Error>
Calculates the expectation value <psi|H|psi> = Sum_i <psi|P_i|psi>.
The expectation value is generally real for Hermitian operators and normalised states.
However, this function returns a Complex<f64> as intermediate PauliStrings
might have complex coefficients or the operator/state might not be strictly physical.
Arguments
state- The state |psi> for which to calculate the expectation value. For a physically meaningful expectation value, this state should be normalised.
Returns
Result<Complex<f64>, Error>- The expectation value, or an error if the operation fails.
Errors
- Returns an error if any underlying
PauliString::applyfails (eg., invalid qubit index). - Returns an error if
state.inner_productfails (eg., mismatched number of qubits, thoughPauliString::applyshould also catch qubit count issues).