pub struct Subroutine {
pub gates: Vec<Gate>,
pub num_qubits: usize,
}Expand description
A subroutine for a quantum circuit.
Fields
-
gates- A vector of gates in the subroutine. -
num_qubits- The number of qubits in the subroutine.
Fields
gates: Vec<Gate>The gates in the subroutine.
num_qubits: usizeThe number of qubits in the subroutine.
Implementations
Sourceimpl Subroutine
impl Subroutine
Sourcepub fn new(num_qubits: usize) -> Self
pub fn new(num_qubits: usize) -> Self
Creates a new subroutine with the specified number of qubits.
Arguments
num_qubits- The number of qubits in the subroutine.
Sourcepub fn with_gates(gates: Vec<Gate>, num_qubits: usize) -> Subroutine
pub fn with_gates(gates: Vec<Gate>, num_qubits: usize) -> Subroutine
Creates a new subroutine with the specified gates and number of qubits.
Arguments
gates- A vector of gates in the subroutine.num_qubits- The number of qubits in the subroutine.
Sourcepub fn get_gates(&self) -> &Vec<Gate>
pub fn get_gates(&self) -> &Vec<Gate>
Gets the gates in the subroutine.
Returns
&Vec<Gate>- A reference to the vector of gates in the subroutine.
Sourcepub fn add_gates(&mut self, gates: Vec<Gate>)
pub fn add_gates(&mut self, gates: Vec<Gate>)
Adds multiple gates to the subroutine.
Arguments
gates- A vector of gates to be added to the subroutine.
Sourcepub fn get_num_qubits(&self) -> usize
pub fn get_num_qubits(&self) -> usize
Gets the number of qubits in the subroutine.
Returns
usize- The number of qubits in the subroutine.
Sourcepub fn qft(qubits: Vec<usize>, num_qubits: usize) -> Subroutine
pub fn qft(qubits: Vec<usize>, num_qubits: usize) -> Subroutine
Creates a quantum fourier transform subroutine for the specified qubits.
Arguments
-
qubits- A vector of indices of the qubits to be transformed. -
num_qubits- The number of qubits in the subroutine.