pub struct Circuit {
pub gates: Vec<Gate>,
pub num_qubits: usize,
}Expand description
Represents a quantum circuit as a vector of gates.
Fields
gates- A vector of gates in the circuit.num_qubits- The number of qubits in the circuit.
Fields
gates: Vec<Gate>A vector of gates in the circuit.
num_qubits: usizeThe number of qubits in the circuit.
Implementations
Sourceimpl Circuit
impl Circuit
Sourcepub fn get_num_qubits(&self) -> usize
pub fn get_num_qubits(&self) -> usize
Sourcepub fn get_gates(&self) -> &Vec<Gate>
pub fn get_gates(&self) -> &Vec<Gate>
Gets the gates in the circuit.
Returns
&Vec<Gate>- A reference to the vector of gates in the circuit.
Sourcepub fn execute(&self, initial_state: &State) -> Result<State, Error>
pub fn execute(&self, initial_state: &State) -> Result<State, Error>
Executes the circuit with the given initial state, and returns the final state.
Arguments
initial_state- The initial state of the qubits in the circuit.
Returns
Result<State, Error>- The final state of the qubits after executing the circuit.
Errors
- Returns an error if the number of qubits in the initial state does not match the number of qubits in the circuit.
- Returns an error if the circuit cannot be executed due to invalid gate operations.
Sourcepub fn trace_execution(
&self,
initial_state: &State,
) -> Result<Vec<State>, Error>
pub fn trace_execution( &self, initial_state: &State, ) -> Result<Vec<State>, Error>
Executes the circuit with the given initial state, and returns all the intermediate states and the final state.
Arguments
initial_state- The initial state of the qubits in the circuit.
Returns
Result<Vec<State>, Error>- A vector of intermediate states and the final state after executing the circuit.
Errors
- Returns an error if the number of qubits in the initial state does not match the number of qubits in the circuit.
- Returns an error if the circuit cannot be executed due to invalid gate operations.