pub struct Unitary2 { /* private fields */ }Expand description
An arbitrary 2×2 unitary operator.
This operator can be applied to a single qubit in a quantum state. It is represented by a 2×2 unitary matrix.
Implementations
Sourceimpl Unitary2
impl Unitary2
Sourcepub fn new(matrix: [[Complex<f64>; 2]; 2]) -> Result<Self, Error>
pub fn new(matrix: [[Complex<f64>; 2]; 2]) -> Result<Self, Error>
Creates a new Unitary2 operator with the given 2×2 unitary matrix.
Arguments:
matrix- A 2×2 unitary matrix represented as a 2D array of complex numbers.
Returns:
Result<Self, Error>- A result containing the new Unitary2 operator or an error if the matrix is not unitary.
Errors:
Error::NonUnitaryMatrix- If the provided matrix is not unitary.