pub struct Parameter<const N: usize> { /* private fields */ }Expand description
A struct representing a set of mutable parameters for a parametrised gate. Cloning this struct is cheap and allows multiple gates to share the same underlying parameters.
Fields
values: AnArc<Mutex<[f64; N]>>that holds the parameter values.
Generic Parameters
N: The number of values in the parameter array.
Implementations
Sourceimpl<const N: usize> Parameter<N>
impl<const N: usize> Parameter<N>
Sourcepub fn deep_clone(&self) -> Self
pub fn deep_clone(&self) -> Self
Creates a new Parameter instance as an independent copy of the given parameter.
Returns
A new Parameter instance.